@@ -6,7 +6,11 @@ use utf8;
 
 use POSIX qw(_exit setsid :sys_wait_h);
 use IO::Handle;
-use KVM::Kavoom;
+use FindBin;
+use File::Basename qw(dirname);
+use KVM::Kavoom::Config::Global;
+use KVM::Kavoom::Config::Instance;
+use KVM::Kavoom::Instance;
 
 our $PACKAGE //= 'kavoom';
 our $VERSION //= 'git';
@@ -15,16 +19,9 @@ our $sysconfdir;
 
 my $exit_status;
 
-sub kvm {
-	my $name = shift;
-	die "missing kvm name\n" unless defined $name;
-	my $kvm = new KVM::Kavoom($name);
-	$kvm->config;
-	return $kvm;
-}
-
 my %tried;
 my @tried;
+my $configdir;
 my $configfile;
 
 sub concat {
@@ -33,32 +30,47 @@ sub concat {
 }
 
 foreach(
-			$ENV{KAVOOMRC},
+			concat($ENV{KAVOOMRC}),
 			concat($ENV{HOME}, '.kavoomrc'),
 			concat($sysconfdir, 'kavoom.cfg'),
-			concat($prefix, 'etc', 'kavoom.cfg'),
+			concat($prefix, 'etc/kavoom.cfg'),
+			concat(dirname($FindBin::Bin), 'etc/kavoom.cfg'),
 			'/etc/kavoom.cfg',
 			'/usr/local/etc/kavoom.cfg',
-			"/opt/$PACKAGE/etc/kavoom.cfg",
 			"/opt/$PACKAGE-$VERSION/etc/kavoom.cfg",
+			"/opt/$PACKAGE/etc/kavoom.cfg",
 		) {
-	next unless defined;
 	next if exists $tried{$_};
 	undef $tried{$_};
 	push @tried, $_;
 	next unless -e;
+	unless(-f _) {
+		warn "kavoom: skipping $_ (not a file)\n";
+		next;
+	}
 	unless(-r _) {
-		warn "Skipping unreadable $_\n";
+		warn "kavoom: skipping $_ (unreadable)\n";
 		next;
 	}
 	$configfile = $_;
+	$configdir = $_;
+	$configdir =~ s{/[^/]+$}{}a;
+	$configdir = '.' if $configdir eq '';
 	last;
 }
 
-die "Can't find a configuration file. Tried:\n".join('', map {"\t$_\n"} @tried)
+die "can't find a configuration file. Tried:\n".join('', map {"\t$_\n"} @tried)
 	unless defined $configfile;
 
-KVM::Kavoom::configure($configfile);
+my $globalcfg = new KVM::Kavoom::Config::Global(configdir => $configdir, load => $configfile);
+
+sub kvm {
+	my $name = shift;
+	die "missing kvm name\n" unless defined $name;
+	my $configdir = $globalcfg->configdir;
+	my $cfg = new KVM::Kavoom::Config::Instance(template => $globalcfg, load => "$configdir/$name.cfg");
+	return new KVM::Kavoom::Instance(name => $name, config => $cfg);
+}
 
 sub start_or_resume {
 	my $resume = shift;
@@ -337,6 +349,9 @@ Usually F</usr/bin/kvm> or F<kvm>.
 
 May be overridden by a vm configuration file.
 
+You may also specify VM options in this file (see below). These will serve
+as defaults for all VMs.
+
 =back
 
 =head1 CONFIGURATION
@@ -489,7 +504,7 @@ Wessel Dankers <wsl@fruit.je>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2009 Wessel Dankers <L<wsl@fruit.je|mailto:wsl@fruit.je>>
+Copyright (c) 2009-2017 Wessel Dankers <L<wsl@fruit.je>>
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
